//
// Marina Drosou (mdrosou@cs.uoi.gr)
// University of Ioannina
//
// PrefSIENA Publisher
//
// Last update: 12 Dec 2008
//

import java.util.Vector;
import java.util.Iterator;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.StringTokenizer;


import java.util.Date;
import javax.management.timer.Timer;

import siena.*;

public class Publisher {

	//
	// mdrosou - read publications (movies mode)
	//
	public static Vector readData_Movies(String filename) {
		Vector notifications = new Vector();
		int num = 0;
		
		try{
			// Read publications from file and create the corresponding notifications
			FileReader inFile = new FileReader(filename);
			BufferedReader bufRead = new BufferedReader(inFile);
			
	        String line;
			Notification e = new Notification();

			while((line = bufRead.readLine()) != null) {
				
				//System.out.println(line);
				
				if(line.equals("")) {
					num++;
					e.putAttribute("Num", num);
					notifications.add(e);
					e = new Notification();
					continue;
				}
				
				String tokens[] = {"",""};
				int count = 0;
				
				StringTokenizer st = new StringTokenizer(line, "|");
				while (st.hasMoreTokens())
					tokens[count++] = st.nextToken();	
								
				if(tokens[0].equals("title") || tokens[0].equals("genre") || tokens[0].equals("mpaa") || tokens[0].equals("att"))
					e.putAttribute(tokens[0], tokens[1]);
				else if(tokens[0].equals("rating"))
					e.putAttribute(tokens[0], Double.parseDouble(tokens[1]));
				else
					e.putAttribute(tokens[0], Integer.parseInt(tokens[1]));
	        }
	            
			bufRead.close();
		}catch (Exception e){ 
			System.err.println("Error: " + e.getMessage());
			System.err.println("Num: " + num);
		}
		
		return notifications;
	}

	//
	// mdrosou - read publications (flickr mode)
	//
	public static Vector readData_Flickr(String filename) {
		Vector notifications = new Vector();
		
		try{
			// Read publications from file and create the corresponding notifications
			FileReader inFile = new FileReader(filename);
			BufferedReader bufRead = new BufferedReader(inFile);
			
	        String line;
			Notification e = new Notification();
			String attributes[] = {"Index", "ID", "NSID", "Title", "Taken", "Posted", "URL", "Tags"};
			int indx = 0;

			
			while((line = bufRead.readLine()) != null) {				

				String[] result = line.split("\\|");
				
				for (int i=0; i<result.length; i++)
					if(!result[i].equals(null))
						e.putAttribute(attributes[i], result[i]);

				notifications.add(e);
				//System.out.println(e.toString());
				e = new Notification();
			}
	            
			bufRead.close();
		}catch (Exception e){ 
			System.err.println("Error: " + e.getMessage());
		}
		
		return notifications;
	}
	
	//
	// publishes data to the system
	//
    public static void main(String[] args) {
		if(args.length != 3) {
			System.out.println("Usage: ?>Publisher [master server] [notifications_file] [intra-publication time]");
			System.exit(1);
		}	
		
		try {
			// start client
			Publisher party = new Publisher();	
		
			// define master server
			ThinClient siena = new ThinClient("ka:localhost:2345");			
			
			// read publications
			// Vector notifications = readData_Flickr(args[1]);			
			Vector notifications = readData_Movies(args[1]);	
			
	/*		Iterator si;
			for(si = notifications.iterator(); si.hasNext();) {
				Notification ee = new Notification((Notification)si.next());
				System.out.println("\npublishing:");
				System.out.println(ee.toString());	
			} */
			
			Thread.sleep(1000); 
			
			try {				
				Iterator si;
				int ctr = 1;
				for(si = notifications.iterator(); si.hasNext();) {
					Thread.sleep(Integer.parseInt(args[2]));
					Notification ee = new Notification((Notification)si.next());
					
					// System.out.println("\n(" + ctr++ + ") publishing:");
					// System.out.println(ee.toString());	
					siena.publish(ee);
				}
		    } catch (SienaException ex) {
				System.err.println("Siena error:" + ex.toString());
		    }
		    
			System.out.println("shutting down.");
		    siena.shutdown();			
		} catch (Exception ex) {
		    ex.printStackTrace();
		    System.exit(1);
		} 
    };
}
