//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Antonio Carzaniga <carzanig@cs.colorado.edu>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 1998-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: InetPacketReceiver.java,v 1.3 2002/07/05 21:45:49 carzanig Exp $
//
package siena.comm;

import java.net.InetAddress;
import java.net.Socket;
import java.net.ServerSocket;
import java.net.DatagramSocket;
import java.io.IOException;
import java.net.UnknownHostException;

/** base class for IP-based packet receivers
 **/
abstract class InetPacketReceiver {
    static final String		Separator = ":";
    byte[]			my_address;

    /** 
     *  This method has been replaced by {@link #address()}.  The
     *  reason for this change is more conceptual than practical: the
     *  term "address" is more appropriate than the term "uri" because
     *  it provides a way to reach a <code>PacketReceiver</code>,
     *  rather than an identifier.
     *
     *  @deprecated since 1.4.0.  Please use {@link
     *  PacketReceiver#address() address()} instead.
     **/
    public byte[] uri() {
	return my_address;
    }

    /** external address of this packet receiver.
     *
     *  Uses the following schema syntax:<br>
     *
     *  <em>schema</em><code>:</code><em>host</em><code>:</code><em>port</em>
     * 
     *  @see PacketReceiver#address()
     **/
    public byte[] address() {
	return my_address;
    }

    protected static InetAddress guessMyIPAddress() 
	throws UnknownHostException {
	return InetAddress.getLocalHost();
    }

    protected static InetAddress guessMyIPAddress(ServerSocket s) 
    throws UnknownHostException {
	InetAddress res = s.getInetAddress();
	if (res != null) {
	    byte [] addr = res.getAddress();
	    if (addr[0] != 0 || addr[1] != 0 || addr[2] != 0 || addr[3] != 0)
		return res;
	}
	return InetAddress.getLocalHost();
    }

    protected static InetAddress guessMyIPAddress(DatagramSocket s) 
    throws UnknownHostException {
	InetAddress res = s.getInetAddress();
	if (res != null) {
	    byte [] addr = res.getAddress();
	    if (addr[0] == 0 &&  addr[1] == 0 && addr[2] == 0 &&  addr[3] == 0)
		return res;
	}
	return InetAddress.getLocalHost();
    }
}
