//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Antonio Carzaniga <carzanig@cs.colorado.edu>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 1998-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: KAPacketSender.java,v 1.4 2002/12/12 22:43:39 carzanig Exp $
//
package siena.comm;

import java.net.InetAddress;
import java.net.Socket;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;

class KAPacketSender extends InetPacketSender implements PacketSender {
    private Socket		socket;
    private OutputStream	os;
    private InputStream		is;
    private long		last_conn = -1;

    static public int		DefaultSoTimeout = 5000;
    private int			so_timeout = DefaultSoTimeout;

    static public int		DefaultDisconnectTimeout = 5000;
    private int			disconnect_timeout = DefaultSoTimeout;

    String schema() { return KAPacketReceiver.Schema; }
    int default_port() { return KAPacketReceiver.DefaultPort; }

    public KAPacketSender(String h) 
	throws InvalidSenderException, java.net.UnknownHostException {
	super(h);
    }

    Socket connect_with_ok() throws IOException {
	Socket s;
	do {
	    s = new Socket(ip_address, port);
	    if (s.getInputStream().read() != 0) {
		s.close();
		s = null;
	    } else {
		s.setTcpNoDelay(true);
		s.shutdownInput();
		s.setSoTimeout(so_timeout);
		s.setSendBufferSize(65536);
		return s;
	    }
	} while (true);
    }

    Socket connect() throws IOException {
	Socket s = new Socket(ip_address, port);
	s.setTcpNoDelay(true);
	s.setSoTimeout(so_timeout);
	s.setSendBufferSize(65536);
	return s;
    }

    public void send(byte[] packet, int offset, int len) 
	throws PacketSenderException {
	int retries;
	retries = 2;
	do {
	    try {
		if (socket == null) {
		    try {
			socket = this.connect();
		    } catch (IOException ex) {
			throw new PacketSenderException(ex.getMessage());
		    }
		    os = socket.getOutputStream();
		    is = socket.getInputStream();
		}
		//
		// here I'm attempting to send the packet.  But before
		// I do that I check that the connection has not been
		// closed by the receiver.  The receiver does that by
		// writing one byte back to me (the sender), so what I
		// do is simply to check that is.available() == 0
		// before sending.
		//
		if (is.available() == 0) {
		    synchronized (os) {
			os.write((len & 0xff00) >>> 8);
			os.write(len & 0xff);
			if (len > 0)
			    os.write(packet, offset, len);
			os.flush();
		    }
		    return;
		} else {
		    //
		    // if the receiver has closed this connection, I
		    // close this end of the connection and iterate as
		    // usual (see retries variable).
		    //
		    shutdown();
		    --retries;
		}
	    } catch (IOException ex) {
		shutdown();
		if (--retries == 0)
		    throw new PacketSenderException(ex.toString());
	    }
	} while (retries > 0); 
	throw new PacketSenderException("connector closed");
    }
    
    public void shutdown() {
	if (socket != null) 
	    try {
		os.close();
		is.close();
		socket.close();
	    } catch (IOException closex) { 
		//
		// how should I deal with this?
		//
	    }
	socket = null;
	os = null;
	is = null;
    }

    public void send(byte[] packet) throws PacketSenderException {
	send(packet, 0, packet.length);
    }

    public void send(byte[] packet, int len) throws PacketSenderException {
	send(packet, 0, len);
    }
}
